// generated by CommonLibrary.cmake, do not edit.

/**
 * @file include/hwsd/version.h
 * Defines version macros and class for library hwsd.
 */

#ifndef HWSD_VERSION_H
#define HWSD_VERSION_H

#include <hwsd/api.h>
#include <string>

namespace hwsd
{
    /** The current major version. */
#   define HWSD_VERSION_MAJOR 2

    /** The current minor version. */
#   define HWSD_VERSION_MINOR 0

    /** The current patch level. */
#   define HWSD_VERSION_PATCH 1

    /** The current SCM revision. */
#   define HWSD_VERSION_REVISION 0x0

    /** The current binary interface. */
#   define HWSD_VERSION_ABI 4ull

    /** The current major.minor.version as a string. */
#   define HWSD_VERSION_STRING \
    "2.0.1"

    /** The current major.minor.version.revision as a string. */
#   define HWSD_REV_STRING \
    "2.0.1.0x0"

/** True if the current version is newer than the given one. */
#   define HWSD_VERSION_GT( MAJOR, MINOR, PATCH )       \
    ( (HWSD_VERSION_MAJOR>MAJOR) ||                     \
      (HWSD_VERSION_MAJOR==MAJOR && (HWSD_VERSION_MINOR>MINOR || \
          (HWSD_VERSION_MINOR==MINOR && HWSD_VERSION_PATCH>PATCH))))

/** True if the current version is equal or newer to the given. */
#   define HWSD_VERSION_GE( MAJOR, MINOR, PATCH )       \
    ( (HWSD_VERSION_MAJOR>MAJOR) ||                     \
      (HWSD_VERSION_MAJOR==MAJOR && (HWSD_VERSION_MINOR>MINOR || \
          (HWSD_VERSION_MINOR==MINOR && HWSD_VERSION_PATCH>=PATCH))))

/** True if the current version is older than the given one. */
#   define HWSD_VERSION_LT( MAJOR, MINOR, PATCH )       \
    ( (HWSD_VERSION_MAJOR<MAJOR) ||                     \
      (HWSD_VERSION_MAJOR==MAJOR && (HWSD_VERSION_MINOR<MINOR || \
          (HWSD_VERSION_MINOR==MINOR && HWSD_VERSION_PATCH<PATCH))))

/** True if the current version is older or equal to the given. */
#   define HWSD_VERSION_LE( MAJOR, MINOR, PATCH )       \
    ( (HWSD_VERSION_MAJOR<MAJOR) ||                     \
      (HWSD_VERSION_MAJOR==MAJOR && (HWSD_VERSION_MINOR<MINOR || \
        (HWSD_VERSION_MINOR==MINOR && HWSD_VERSION_PATCH<=PATCH))))

/** Information about the current hwsd version. */
class HWSD_API Version
{
public:
    /** @return the current major version of hwsd. */
    static int getMajor();

    /** @return the current minor version of hwsd. */
    static int getMinor();

    /** @return the current patch level of hwsd. */
    static int getPatch();

    /** @return the current hwsd version (MM.mm.pp). */
    static std::string getString();

    /** @return the SCM revision. */
    static unsigned long long getRevision();

    /** @return the current binary interface version of hwsd. */
    static int getABI();

    /** @return the current hwsd version plus the git SHA hash (MM.mm.pp.rev). */
    static std::string getRevString();

    /**
     * Runtime check for ABI compatibility.
     *
     * Call from code using hwsd. Will fail if the executable
     * was compiled against a version incompatible with the runtime version.
     *
     * @return true if the link-time and compile-time DSO are compatible.
     */
    static bool check()
    {
        return getMajor()==HWSD_VERSION_MAJOR &&
               getMinor()==HWSD_VERSION_MINOR;
    }

    /** @return JSON schema describing this object. */
    static std::string getSchema()
    {
        return R"(
                {
                  "title": "Version",
                  "type": "object",
                  "description": "Version information of the running application",
                  "properties": {
                    "major": { "type": "integer" },
                    "minor": { "type": "integer" },
                    "patch": { "type": "integer" },
                    "abi": { "type": "integer" },
                    "revision": { "type": "string" }
                  }
               }
               )";
    }

    /** @return JSON-encoded data for this object. */
    static std::string toJSON()
    {
        return R"(
               {
                 "major": 2,
                 "minor": 0,
                 "patch": 1,
                 "abi": 4,
                 "revision": "0"
               }
               )";
    }
};

}

#endif
